package uk.ac.gla.photofight.personalize;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import uk.ac.gla.photofight.R;
import uk.ac.gla.photofight.Storage;
import android.app.Activity;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.hardware.Camera;
import android.hardware.Camera.PictureCallback;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.Toast;

public class TakePhotoActivity extends Activity {
	
	private String type;
	private String fighterName;

	private Camera mCamera;
	private CameraPreview mPreview;
	private PictureCallback mPicture;
	
	private BitmapFactory.Options options;

	@Override
	protected void onPause() {
		super.onPause();
		releaseCamera(); // release the camera immediately on pause event
	}

	private void releaseCamera() {
		if (mCamera != null) {
			mCamera.release(); // release the camera for other applications
			mCamera = null;
		}
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.take_photo);

		type = getIntent().getExtras().getString("type");
		Toast.makeText(TakePhotoActivity.this, type.toUpperCase() + "!", Toast.LENGTH_LONG).show();
		
		SharedPreferences settings = getSharedPreferences("photoFight", 0);
		fighterName = settings.getString("fighterName", null);
		
		options = new BitmapFactory.Options();
		options.inSampleSize = 8;

		mPicture = new PictureCallback() {

			@Override
			public void onPictureTaken(byte[] data, Camera camera) {
				File pictureFile = Storage.getOutputMediaFile(TakePhotoActivity.this, fighterName, type + ".png");
				if (pictureFile == null) {
					Log.d("pictureFile create",
							"Error creating media file, check storage permissions");
					return;
				}
				try {
					FileOutputStream fos = new FileOutputStream(pictureFile);
					
					byte[] convertedPhoto = removeBackground(data);
					
					fos.write(convertedPhoto);
					fos.close();
					finish();
				} catch (FileNotFoundException e) {
					Log.d("pictureFile create",
							"File not found: " + e.getMessage());
				} catch (IOException e) {
					Log.d("pictureFile create",
							"Error accessing file: " + e.getMessage());
				}
			}
		};

		// Create an instance of Camera
		mCamera = getCameraInstance();

		// get Camera parameters
		Camera.Parameters cparams = mCamera.getParameters();
		cparams.setJpegQuality(10);
		cparams.setFlashMode(Camera.Parameters.FLASH_MODE_AUTO);
		cparams.setRotation(90);
		// set Camera parameters
		mCamera.setParameters(cparams);

		mCamera.setDisplayOrientation(90);

		// Create our Preview view and set it as the content of our activity.
		mPreview = new CameraPreview(this, mCamera);
		FrameLayout preview = (FrameLayout) findViewById(R.id.camera_preview);
		preview.addView(mPreview);
		
		Bitmap borderBitmap = getBorderBitmap();
		float scaleWidth = ((float) cparams.getPreviewSize().height)
				/ borderBitmap.getWidth();
		float scaleHeight = ((float) cparams.getPreviewSize().width + 90)
				/ borderBitmap.getHeight();
		Log.i("Boreder scale", "Width: " + scaleWidth + ", Height: "
				+ scaleHeight);
		Matrix scaleMatrix = new Matrix();
		scaleMatrix.postScale(scaleWidth, scaleHeight);

		Log.i("Camera Preview size", "Width: " + cparams.getPreviewSize().width
				+ ", Height: " + cparams.getPreviewSize().height);
		Bitmap scaledBitmap = Bitmap.createBitmap(
				cparams.getPreviewSize().height,
				cparams.getPreviewSize().width + 90, borderBitmap.getConfig());
		Canvas canvas = new Canvas(scaledBitmap);
		canvas.drawBitmap(borderBitmap, scaleMatrix, null);

		View borderView = getLayoutInflater().inflate(R.layout.photo_border,
				null);
		ImageView borderImageView = (ImageView) borderView
				.findViewById(R.id.photo_border_imageview);
		borderImageView.setImageBitmap(scaledBitmap);
		borderImageView.invalidate();
		// LayoutParams lparams = new
		// LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
		// ViewGroup.LayoutParams.MATCH_PARENT);
		preview.addView(borderView);

		// Add a listener to the Capture button
		Button captureButton = (Button) findViewById(R.id.button_capture);
		captureButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				// get an image from the camera
				mCamera.takePicture(null, null, mPicture);
			}
		});

	}
	
	private byte[] removeBackground(byte[] data) {
		Bitmap photo = BitmapFactory.decodeByteArray(data, 0, data.length, options);
		Bitmap background = getBackgroundBitmap();
		
		float scaleWidth = ((float) background.getWidth()) / photo.getWidth();
		float scaleHeight = ((float) background.getHeight()) / photo.getHeight();
		
		Matrix scaleMatrix = new Matrix();
		scaleMatrix.postScale(scaleWidth, scaleHeight);

		Bitmap newBitmap = Bitmap.createBitmap(background.getWidth(),
				background.getHeight(), background.getConfig());
		
		Paint paint = new Paint();
		paint.setFilterBitmap(false);

		Canvas canvas = new Canvas(newBitmap);
		canvas.drawBitmap(photo, scaleMatrix, paint);
		paint.setXfermode(new PorterDuffXfermode(PorterDuff.Mode.DST_IN));
		canvas.drawBitmap(background, new Matrix(), paint);
		paint.setXfermode(null);

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		newBitmap.compress(CompressFormat.PNG, 0, bos);
		byte[] newBitmapData = bos.toByteArray();
		
		return newBitmapData;
	}
	
	private Bitmap getBorderBitmap() {
		
		Bitmap result;
		if (type.equals("walk"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.borderwalk, options);
		else if (type.equals("stand"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.borderstand, options);
		else if (type.equals("punch"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.borderpunch, options);
		else if (type.equals("kick"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.borderkick, options);
		else if (type.equals("take_hit"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bordertake_hit, options);
		else if (type.equals("crouch"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bordercrouch, options);
		else if (type.equals("victory"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bordervictory_defeat, options);
		else if (type.equals("defeat"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bordervictory_defeat, options);
		else 
			result = null;
		return result;
	}
	
	private Bitmap getBackgroundBitmap() { 
		
		Bitmap result;
		if (type.equals("walk"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bgwalk, options);
		else if (type.equals("stand"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bgstand, options);
		else if (type.equals("punch"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bgpunch, options);
		else if (type.equals("kick"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bgkick, options);
		else if (type.equals("take_hit"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bgtake_hit, options);
		else if (type.equals("crouch"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bgcrouch, options);
		else if (type.equals("victory"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bgvictory_defeat, options);
		else if (type.equals("defeat"))
			result = BitmapFactory.decodeResource(getResources(), R.drawable.bgvictory_defeat, options);
		else
			result = null;
		return result;
	}

	// Taken from the Android developer guide
	public static Camera getCameraInstance() {
		Camera c = null;
		try {
			c = Camera.open(); // attempt to get a Camera instance
		} catch (Exception e) {
			// Camera is not available (in use or does not exist)
		}
		return c; // returns null if camera is unavailable
	}



}
